import { render, screen } from '@testing-library/react'
import { userEvent } from '@testing-library/user-event'
import { DataloadState, FluidType } from 'enums'
import { DateTime } from 'luxon'
import React from 'react'
import { Provider } from 'react-redux'
import * as storeHooks from 'store/hooks'
import { baseDataLoad } from 'tests/__mocks__/chartData.mock'
import { createMockEcolyoStore } from 'tests/__mocks__/store'
import InfoDataConsumptionVisualizer from './InfoDataConsumptionVisualizer'

const mockAppDispatch = jest.spyOn(storeHooks, 'useAppDispatch')

describe('InfoDataConsumptionVisualizer component', () => {
  const mockLastDataDate = DateTime.fromISO('2020-10-01T00:00:00.000Z', {
    zone: 'utc',
  })

  const store = createMockEcolyoStore()

  it('should render correctly when data valid', () => {
    const mockDataLoad = { ...baseDataLoad, state: DataloadState.UPCOMING }
    const { container } = render(
      <Provider store={store}>
        <InfoDataConsumptionVisualizer
          dataload={mockDataLoad}
          fluidType={FluidType.ELECTRICITY}
          lastDataDate={mockLastDataDate}
        />
      </Provider>
    )
    expect(container).toMatchSnapshot()
  })

  describe('should render correctly consumption_visualizer.last_available_data', () => {
    it('case state MISSING', async () => {
      const mockDataLoad = { ...baseDataLoad, state: DataloadState.MISSING }
      render(
        <Provider store={store}>
          <InfoDataConsumptionVisualizer
            dataload={mockDataLoad}
            fluidType={FluidType.ELECTRICITY}
            lastDataDate={mockLastDataDate}
          />
        </Provider>
      )
      expect(
        await screen.findByText('consumption_visualizer.last_available_data')
      ).toBeInTheDocument()
    })
    it('case state UPCOMING', async () => {
      const mockDataLoad = { ...baseDataLoad, state: DataloadState.UPCOMING }
      render(
        <Provider store={store}>
          <InfoDataConsumptionVisualizer
            dataload={mockDataLoad}
            fluidType={FluidType.ELECTRICITY}
            lastDataDate={mockLastDataDate}
          />
        </Provider>
      )
      expect(
        await screen.findByText('consumption_visualizer.last_available_data')
      ).toBeInTheDocument()
    })
    it('case state COMING', async () => {
      const mockDataLoad = {
        ...baseDataLoad,
        state: DataloadState.COMING,
      }
      render(
        <Provider store={store}>
          <InfoDataConsumptionVisualizer
            dataload={mockDataLoad}
            fluidType={FluidType.ELECTRICITY}
            lastDataDate={mockLastDataDate}
          />
        </Provider>
      )
      expect(
        await screen.findByText('consumption_visualizer.last_available_data')
      ).toBeInTheDocument()
    })
    it('case state AGGREGATED_HOLE_OR_MISSING', async () => {
      const mockDataLoad = {
        ...baseDataLoad,
        state: DataloadState.AGGREGATED_HOLE_OR_MISSING,
      }
      render(
        <Provider store={store}>
          <InfoDataConsumptionVisualizer
            dataload={mockDataLoad}
            fluidType={FluidType.MULTIFLUID}
            lastDataDate={mockLastDataDate}
          />
        </Provider>
      )
      expect(
        await screen.findByText('consumption_visualizer.last_valid_data_multi')
      ).toBeInTheDocument()
    })
    it('case state AGGREGATED_WITH_HOLE_OR_MISSING', async () => {
      const mockDataLoad = {
        ...baseDataLoad,
        state: DataloadState.AGGREGATED_WITH_HOLE_OR_MISSING,
      }
      render(
        <Provider store={store}>
          <InfoDataConsumptionVisualizer
            dataload={mockDataLoad}
            fluidType={FluidType.MULTIFLUID}
            lastDataDate={mockLastDataDate}
          />
        </Provider>
      )
      expect(
        await screen.findByText('consumption_visualizer.last_valid_data_multi')
      ).toBeInTheDocument()
    })
    it('case state AGGREGATED_COMING', async () => {
      const mockDataLoad = {
        ...baseDataLoad,
        state: DataloadState.AGGREGATED_COMING,
      }
      render(
        <Provider store={store}>
          <InfoDataConsumptionVisualizer
            dataload={mockDataLoad}
            fluidType={FluidType.MULTIFLUID}
            lastDataDate={mockLastDataDate}
          />
        </Provider>
      )
      expect(
        await screen.findByText('consumption_visualizer.last_valid_data_multi')
      ).toBeInTheDocument()
    })
  })

  describe('should render correctly with why_no_data', () => {
    it('case state EMPTY', async () => {
      const mockDataLoad = { ...baseDataLoad, state: DataloadState.EMPTY }
      render(
        <Provider store={store}>
          <InfoDataConsumptionVisualizer
            dataload={mockDataLoad}
            fluidType={FluidType.ELECTRICITY}
            lastDataDate={mockLastDataDate}
          />
        </Provider>
      )
      expect(
        await screen.findByText('consumption_visualizer.why_no_data')
      ).toBeInTheDocument()
    })
    it('case state HOLE', async () => {
      const mockDataLoad = { ...baseDataLoad, state: DataloadState.HOLE }
      render(
        <Provider store={store}>
          <InfoDataConsumptionVisualizer
            dataload={mockDataLoad}
            fluidType={FluidType.ELECTRICITY}
            lastDataDate={mockLastDataDate}
          />
        </Provider>
      )
      expect(
        await screen.findByText('consumption_visualizer.why_no_data')
      ).toBeInTheDocument()
    })
    it('case state AGGREGATED_EMPTY', async () => {
      const mockDataLoad = {
        ...baseDataLoad,
        state: DataloadState.AGGREGATED_EMPTY,
      }
      render(
        <Provider store={store}>
          <InfoDataConsumptionVisualizer
            dataload={mockDataLoad}
            fluidType={FluidType.MULTIFLUID}
            lastDataDate={mockLastDataDate}
          />
        </Provider>
      )
      expect(
        await screen.findByText('consumption_visualizer.why_no_data')
      ).toBeInTheDocument()
    })
  })

  it('should click on last valid data date and move to it', async () => {
    const mockDataLoad = { ...baseDataLoad, state: DataloadState.EMPTY }
    render(
      <Provider store={store}>
        <InfoDataConsumptionVisualizer
          dataload={mockDataLoad}
          fluidType={FluidType.ELECTRICITY}
          lastDataDate={mockLastDataDate}
        />
      </Provider>
    )
    await userEvent.click(screen.getByRole('button'))
    expect(screen.getByRole('dialog')).toBeInTheDocument()
  })

  it('should click to display NoDataModal', async () => {
    const mockDataLoad = { ...baseDataLoad, state: DataloadState.EMPTY }
    render(
      <Provider store={store}>
        <InfoDataConsumptionVisualizer
          dataload={mockDataLoad}
          fluidType={FluidType.ELECTRICITY}
          lastDataDate={mockLastDataDate}
        />
      </Provider>
    )
    await userEvent.click(screen.getByRole('button'))
    expect(mockAppDispatch).toHaveBeenCalled()
  })
})
