import Button from '@material-ui/core/Button'
import warningDark from 'assets/icons/ico/warning-dark.svg'
import warningWhite from 'assets/icons/ico/warning-white.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { FluidType, KonnectorUpdate } from 'enums'
import { DateTime } from 'luxon'
import {
  Account,
  AccountSgeData,
  FluidConnection,
  FluidStatus,
  Trigger,
} from 'models'
import React, { useCallback, useEffect, useState } from 'react'
import AccountService from 'services/account.service'
import DateChartService from 'services/dateChart.service'
import TriggerService from 'services/triggers.service'
import {
  setShouldRefreshConsent,
  updateFluidConnection,
  updateSgeStore,
} from 'store/global/global.slice'
import { useAppDispatch, useAppSelector } from 'store/hooks'
import { getKonnectorUpdateError } from 'utils/utils'
import DeleteGRDFAccountModal from '../../Connection/GRDFDeleteAccountModal/DeleteGRDFAccountModal'
import './connectionResult.scss'

interface ConnectionResultProps {
  handleAccountDeletion: () => Promise<void>
  fluidType: FluidType
}

const ConnectionResult = ({
  handleAccountDeletion,
  fluidType,
}: ConnectionResultProps) => {
  const { t } = useI18n()
  const client = useClient()
  const dispatch = useAppDispatch()
  const { fluidStatus } = useAppSelector(state => state.ecolyo.global)
  const currentFluidStatus = fluidStatus[fluidType]
  const account: Account | null = currentFluidStatus.connection.account

  const [deleting, setDeleting] = useState<boolean>(false)
  const [updating, setUpdating] = useState<boolean>(false)
  const [lastExecutionDate, setLastExecutionDate] = useState<string | DateTime>(
    '-'
  )
  const [konnectorError, setKonnectorError] = useState<string>('')
  const [status, setStatus] = useState<string>('')
  const [outDatedDataDays, setOutDatedDataDays] = useState<number | null>(null)
  const [openGRDFDeletionModal, setOpenGRDFDeletionModal] =
    useState<boolean>(false)
  const toggleGRDFDeletionModal = useCallback(() => {
    setOpenGRDFDeletionModal(prev => !prev)
  }, [])

  const updateKonnector = async () => {
    setUpdating(true)
    setStatus('')
    setLastExecutionDate('-')
    setKonnectorError('')
    const updatedConnection: FluidConnection = {
      ...currentFluidStatus.connection,
      shouldLaunchKonnector: true,
      isUpdating: true,
    }
    dispatch(
      updateFluidConnection({
        fluidType: currentFluidStatus.fluidType,
        fluidConnection: updatedConnection,
      })
    )
    setUpdating(false)
  }

  const deleteAccountsAndTriggers = useCallback(async () => {
    setDeleting(true)
    try {
      if (account) {
        const accountService = new AccountService(client)
        const triggerService = new TriggerService(client)
        const accounts: Account[] = await accountService.getAccountsByType(
          account.account_type
        )
        for (const _account of accounts) {
          const trigger: Trigger | null =
            await triggerService.getTriggerForAccount(_account)
          if (trigger) await triggerService.deleteTrigger(trigger)
          await accountService.deleteAccount(_account)
        }
        await handleAccountDeletion()
      }
    } catch (error) {
      setDeleting(false)
    }
  }, [account, client, handleAccountDeletion])

  const isOutdated = useCallback(() => {
    const dateChartService = new DateChartService()
    return dateChartService.isDataOutdated(
      currentFluidStatus.lastDataDate,
      currentFluidStatus.fluidType
    )
  }, [currentFluidStatus])

  const hasUpdatedToday = useCallback(() => {
    const todayDate = DateTime.local()
      .setZone('utc', {
        keepLocalTime: true,
      })
      .minus({ days: 1 })
      .toLocaleString()
    if (lastExecutionDate === '-') {
      return false
    } else if (lastExecutionDate > todayDate) {
      return false
    } else {
      return true
    }
  }, [lastExecutionDate])

  const handleRefreshConsent = useCallback(
    (fluidType: FluidType) => {
      if (fluidType == FluidType.ELECTRICITY) {
        const accountData = currentFluidStatus.connection.account
          ?.auth as AccountSgeData
        // store the previous account data since the onDelete will remove account from DB
        dispatch(
          updateSgeStore({
            currentStep: 0,
            firstName: accountData.firstname,
            lastName: accountData.lastname,
            pdl: parseInt(accountData.pointId),
            address: accountData.address,
            zipCode: parseInt(accountData.postalCode),
            city: accountData.city,
            dataConsent: true,
            pdlConfirm: true,
            shouldLaunchAccount: true,
          })
        )
        dispatch(setShouldRefreshConsent(true))
      } else {
        deleteAccountsAndTriggers()
      }
    },
    [
      deleteAccountsAndTriggers,
      dispatch,
      currentFluidStatus.connection.account?.auth,
    ]
  )

  useEffect(() => {
    if (currentFluidStatus.connection.triggerState?.last_success) {
      const result = DateTime.fromISO(
        currentFluidStatus.connection.triggerState.last_success
      )
      setLastExecutionDate(result)
    } else {
      setLastExecutionDate('-')
    }
    if (
      currentFluidStatus.connection.triggerState?.status === 'errored' &&
      currentFluidStatus.connection.triggerState?.last_error
    ) {
      setStatus('errored')
      setKonnectorError(
        getKonnectorUpdateError(
          currentFluidStatus.connection.triggerState.last_error
        )
      )
    }
    if (isOutdated()) {
      setOutDatedDataDays(isOutdated())
    }
  }, [currentFluidStatus.connection.triggerState, isOutdated])

  const getFluidTypeTranslation = (fluidType: FluidType) => {
    switch (fluidType) {
      case FluidType.GAS:
        return 'de gaz'
      case FluidType.ELECTRICITY:
        return "d'électricité"
      default:
        return "d'eau"
    }
  }

  const consentError =
    konnectorError === KonnectorUpdate.ERROR_CONSENT_FORM_GAS ||
    konnectorError === KonnectorUpdate.ERROR_UPDATE_OAUTH

  /**
   * Get Konnector state, possible values:
   * - partner maintenance
   * - error state
   * - outdated
   * - last update date
   */
  const getConnectionStatus = () => {
    // First check if there is partner error from backoffice
    if (currentFluidStatus.maintenance) {
      return (
        <div className="connection-caption text-16-normal">
          <div className="text-16-normal">
            <div className="connection-caption">
              {t('konnector_form.wait_end_issue')}
            </div>
          </div>
        </div>
      )
    }
    // Else check if konnector is in error state
    if (status === 'errored') {
      return (
        <DisplayKonnectorErrorState
          konnectorError={konnectorError}
          consentRelatedError={consentError}
          lastExecutionDate={lastExecutionDate}
          fluidConcerned={getFluidTypeTranslation(fluidType)}
        />
      )
    }
    // Else check if data is outdated
    if (outDatedDataDays) {
      return (
        <DisplayDataOutdated
          fluidStatus={currentFluidStatus}
          fluidType={fluidType}
          lastExecutionDate={lastExecutionDate}
          hasUpdatedToday={hasUpdatedToday()}
        />
      )
    }
    // Default to displaying the last update date
    return (
      <DisplayLastUpdateDate
        fluidType={fluidType}
        lastExecutionDate={lastExecutionDate}
      />
    )
  }

  return (
    <div className="connection-update-result">
      <div
        className={
          status === 'errored' &&
          !hasUpdatedToday() &&
          !currentFluidStatus.maintenance
            ? 'connection-update-errored'
            : ''
        }
      >
        {getConnectionStatus()}
      </div>
      <div className="inline-buttons">
        {!consentError && (
          <Button
            aria-label={t('konnector_form.accessibility.button_disconnect')}
            onClick={
              fluidType === FluidType.GAS
                ? toggleGRDFDeletionModal
                : deleteAccountsAndTriggers
            }
            disabled={updating || deleting}
            className="btnSecondary"
          >
            {deleting
              ? t('konnector_form.loading')
              : t('konnector_form.button_disconnect')}
          </Button>
        )}
        <Button
          aria-label={t('konnector_form.accessibility.button_update')}
          onClick={
            consentError
              ? () => handleRefreshConsent(fluidType)
              : updateKonnector
          }
          disabled={updating || deleting}
          className="btnPrimary"
        >
          {updating && <Loader color="black" />}
          {!updating && (
            <div>
              {consentError
                ? t('konnector_form.button_oauth_reload')
                : t('konnector_form.button_update')}
            </div>
          )}
        </Button>
      </div>
      {fluidType === FluidType.GAS && (
        <DeleteGRDFAccountModal
          open={openGRDFDeletionModal}
          handleCloseClick={toggleGRDFDeletionModal}
          deleteAccount={deleteAccountsAndTriggers}
        />
      )}
    </div>
  )
}

const DisplayKonnectorErrorState = ({
  konnectorError,
  consentRelatedError,
  lastExecutionDate,
  fluidConcerned,
}: {
  konnectorError: string
  consentRelatedError: boolean
  lastExecutionDate: string | DateTime
  fluidConcerned: string
}) => {
  const { t } = useI18n()
  return (
    <div className="connection-caption-errored warning-white text-16-normal">
      <StyledIcon
        icon={warningWhite}
        size={36}
        className="warning-icon"
        role="img"
        title="Attention"
        ariaHidden={false}
      />
      {konnectorError === 'login_failed' ? (
        <div className="text-16-normal">{t('konnector_form.login_failed')}</div>
      ) : (
        <div className="text-16-normal">
          {t(`konnector_form.${konnectorError}`, {
            fluid: fluidConcerned,
          })}
          {!consentRelatedError && (
            <>
              <div className="connection-caption">
                {t('konnector_form.label_updated_at')}
              </div>
              <div className="text-16-bold">
                {lastExecutionDate.toLocaleString()}
              </div>
            </>
          )}
        </div>
      )}
    </div>
  )
}

const DisplayDataOutdated = ({
  fluidStatus,
  fluidType,
  lastExecutionDate,
  hasUpdatedToday,
}: {
  fluidStatus: FluidStatus
  fluidType: FluidType
  lastExecutionDate: string | DateTime
  hasUpdatedToday: boolean
}) => {
  return (
    <div className="connection-caption text-16-normal">
      <div className="text-16-normal">
        <>
          {hasUpdatedToday ? (
            // If user has already ran an update today, display a message about energy provider issue
            <DisplayAlreadyUpdatedToday
              fluidStatus={fluidStatus}
              fluidType={fluidType}
              lastExecutionDate={lastExecutionDate.toLocaleString()}
            />
          ) : (
            <DisplayManualUpdate />
          )}
        </>
      </div>
    </div>
  )
}

/** If user has already ran an update today, display a message about energy provider issue */
const DisplayAlreadyUpdatedToday = ({
  fluidStatus,
  fluidType,
  lastExecutionDate,
}: {
  fluidStatus: FluidStatus
  fluidType: FluidType
  lastExecutionDate: string | DateTime
}) => {
  const { t } = useI18n()

  const getFluidTypeKonnectorTranslation = (fluidType: FluidType) => {
    switch (fluidType) {
      case FluidType.GAS:
        return 'GRDF'
      case FluidType.ELECTRICITY:
        return 'Enedis'
      default:
        return 'Eau Publique du Grand Lyon'
    }
  }
  return (
    <>
      <div className="connection-caption">
        {t('konnector_form.label_updated_at')}
      </div>
      <div className="text-16-bold">{lastExecutionDate.toLocaleString()}</div>
      <div>
        {fluidStatus?.connection?.konnector &&
          t('konnector_form.issue') +
            ' ' +
            getFluidTypeKonnectorTranslation(fluidType) +
            '.'}
      </div>
    </>
  )
}

/** Tells user to run a manual update */
const DisplayManualUpdate = () => {
  const { t } = useI18n()
  return (
    <div className="connection-caption-warning connection-update-errored warning-white text-16-normal">
      <StyledIcon
        icon={warningDark}
        size={36}
        className="warning-icon"
        role="img"
        title="Attention"
        ariaHidden={false}
      />
      <div className="text-16-normal">{t('konnector_form.resolve')}</div>
    </div>
  )
}

const DisplayLastUpdateDate = ({
  lastExecutionDate,
  fluidType,
}: {
  lastExecutionDate: string | DateTime
  fluidType: FluidType
}) => {
  const { t } = useI18n()
  return (
    <div>
      <div className="connection-caption text-16-normal">
        {t('konnector_form.label_updated_at')}
      </div>
      <div className="text-16-bold">{lastExecutionDate.toLocaleString()}</div>
      <br />
      <div className="text-14-normal delta-caption">
        {t(`konnector_form.konnector_delta.${FluidType[fluidType]}`)}
      </div>
    </div>
  )
}

export default ConnectionResult
