import StyledSwitch from 'components/CommonKit/Switch/StyledSwitch'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { FluidState, FluidType, TimeStep } from 'enums'
import { DateTime } from 'luxon'
import { Dataload, TimePeriod } from 'models'
import React, { useCallback, useEffect, useState } from 'react'
import ConsumptionDataManager from 'services/consumption.service'
import { useAppDispatch, useAppSelector } from 'store/hooks'
import { updateProfile } from 'store/profile/profile.slice'
import './reportOptions.scss'

const ReportOptions = () => {
  const { t } = useI18n()
  const client = useClient()
  const dispatch = useAppDispatch()
  const {
    global: { fluidStatus },
    profile,
  } = useAppSelector(state => state.ecolyo)
  const [maxDayData, setLastSemesterMaxDay] = useState<Dataload | null>(null)

  const updateProfileReport = (value: boolean) => {
    dispatch(updateProfile({ sendAnalysisNotification: value }))
  }

  const updateProfileAlert = useCallback(
    (value: boolean) => {
      dispatch(updateProfile({ sendConsumptionAlert: value }))
    },
    [dispatch]
  )

  const isWaterConnected =
    fluidStatus[FluidType.WATER].status !== FluidState.NOT_CONNECTED &&
    fluidStatus[FluidType.WATER].status !== FluidState.KONNECTOR_NOT_FOUND &&
    fluidStatus[FluidType.WATER].status !== FluidState.ERROR_LOGIN_FAILED

  const setWaterLimit = (e: React.ChangeEvent<HTMLInputElement>) => {
    if (e.target.value !== null && parseInt(e.target.value) > 0) {
      dispatch(
        updateProfile({
          waterDailyConsumptionLimit: parseInt(e.target.value),
        })
      )
    } else {
      updateProfileAlert(false)
    }
  }

  const toggleAnalysisNotification = () => {
    profile.sendAnalysisNotification
      ? updateProfileReport(false)
      : updateProfileReport(true)
  }

  const handleAlertChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    e.target.checked ? updateProfileAlert(true) : updateProfileAlert(false)
  }

  useEffect(() => {
    let subscribed = true
    async function getMaxLoadData() {
      const timePeriod: TimePeriod = {
        startDate: DateTime.now().minus({ month: 6 }).startOf('month'),
        endDate: DateTime.now(),
      }
      const consumptionService = new ConsumptionDataManager(client)
      const lastSemesterMax = await consumptionService.getMaxLoad(
        timePeriod,
        TimeStep.DAY,
        [FluidType.WATER],
        undefined,
        false,
        true
      )
      if (lastSemesterMax) {
        setLastSemesterMaxDay(lastSemesterMax as Dataload)
      }
    }
    if (subscribed) {
      getMaxLoadData()
    }
    return () => {
      subscribed = false
    }
  }, [
    client,
    profile.sendConsumptionAlert,
    profile.waterDailyConsumptionLimit,
    updateProfileAlert,
  ])

  return (
    <div className="report-option-root">
      <div className="report-option-content">
        {/* Monthly Report activation */}
        <div className="reportCard">
          <div className="row">
            <StyledSwitch
              onClick={toggleAnalysisNotification}
              checked={profile.sendAnalysisNotification}
            />
            <span className="text-16-normal-uppercase">
              {t('profile.report.title_bilan')}
            </span>
          </div>
          <span className="text-16-normal">
            {t('profile.report.switch_label_bilan')}
          </span>
        </div>
        {/* Consumption Alert activation */}
        {isWaterConnected && (
          <div className="reportCard">
            <div className="row">
              <StyledSwitch
                checked={profile.sendConsumptionAlert}
                onChange={handleAlertChange}
              />

              <div className="text-16-normal-uppercase">
                {t('profile.report.title_alert')}
              </div>
            </div>

            <span className="text-16-normal">
              {t('profile.report.switch_label_alert')}
            </span>

            {profile.sendConsumptionAlert && (
              <>
                <div className="text-16-normal">
                  {t('profile.report.input_label_alert')}
                </div>
                <div className="switch-container-alert">
                  <input
                    className="inputNumber text-18"
                    type="number"
                    defaultValue={
                      profile.waterDailyConsumptionLimit === 0
                        ? ''
                        : profile.waterDailyConsumptionLimit
                    }
                    onBlur={setWaterLimit}
                    aria-label={t(
                      'profile.accessibility.input_water_alert_report'
                    )}
                    inputMode="numeric"
                  />
                  <span className="switch-label text-16-normal">L</span>
                </div>
                {maxDayData && (
                  <div className="alert-input-subtext text-14">
                    {t('profile.report.input_label_subtext_alert')}
                    {Math.round(maxDayData.value)}
                    {' L'}
                  </div>
                )}
              </>
            )}
          </div>
        )}
      </div>
    </div>
  )
}

export default ReportOptions
