import * as Sentry from '@sentry/react'
import { Client, Q, QueryDefinition, QueryResult } from 'cozy-client'
import logger from 'cozy-logger'
import {
  ENEDIS_MAXPOWER_DOCTYPE,
  ENEDIS_MINUTE_DOCTYPE,
  ENEDIS_MONTHLY_ANALYSIS_DATA_DOCTYPE,
  ENEDIS_MONTH_DOCTYPE,
} from 'doctypes'
import { DataloadState, FluidType } from 'enums'
import { DateTime } from 'luxon'
import { Dataload, DataloadEntity, MaxPowerEntity } from 'models'
import {
  AggregatedEnedisMonthlyDataloads,
  EnedisMonthlyAnalysisData,
} from 'models/enedisMonthlyAnalysis'
import logApp from 'utils/logger'
import {
  OffPeakHours,
  formatOffPeakHours,
  parseOffPeakHours,
  roundOffPeakHours,
  splitOffPeakHours,
} from 'utils/utils'
import AccountService from './account.service'
import ConfigService from './fluidConfig.service'

const logStack = logger.namespace('enedisMonthlyAnalysisDataService')
export default class EnedisMonthlyAnalysisDataService {
  private readonly _client: Client

  constructor(_client: Client) {
    this._client = _client
  }

  /**
   * getLastEnedisMonthlyAnalysis
   */
  public async getLastEnedisMonthlyAnalysis(): Promise<
    EnedisMonthlyAnalysisData[]
  > {
    const query: QueryDefinition = Q(ENEDIS_MONTHLY_ANALYSIS_DATA_DOCTYPE)
      .where({})
      .indexFields(['year', 'month'])
      .sortBy([{ year: 'desc' }, { month: 'desc' }])
      .limitBy(1)
    const data = await this._client.query(query)
    return data.data
  }

  /**
   * Aggregates Enedis Analysis data in order to create Dataload injectable in graph component
   */
  public aggregateValuesToDataLoad = (
    data: EnedisMonthlyAnalysisData
  ): AggregatedEnedisMonthlyDataloads => {
    const dataLoadWeekDays: Dataload[] = []
    const dataLoadWeekEndDays: Dataload[] = []
    data.weekDaysHalfHourAverageValues.forEach((value, index) => {
      dataLoadWeekDays.push({
        value: value,
        state: DataloadState.VALID,
        valueDetail: null,
        date: DateTime.fromObject({
          year: data.year,
          month: data.month,
          minute: 0,
        })
          .setZone('utc', {
            keepLocalTime: true,
          })
          .plus({ minute: 30 * index }),
      })
    })
    data.weekEndDaysHalfHourAverageValues.forEach((value, index) => {
      dataLoadWeekEndDays.push({
        value: value,
        state: DataloadState.VALID,
        valueDetail: null,
        date: DateTime.fromObject({
          year: data.year,
          month: data.month,
          minute: 0,
        })
          .setZone('utc', {
            keepLocalTime: true,
          })
          .plus({ minute: 30 * index }),
      })
    })
    return {
      week: dataLoadWeekDays,
      weekend: dataLoadWeekEndDays,
    }
  }

  /**
   * Get an enedis monthly analysis for given month and year
   */
  public async getEnedisMonthlyAnalysisByDate(
    year: number,
    month: number
  ): Promise<EnedisMonthlyAnalysisData[]> {
    const query: QueryDefinition = Q(ENEDIS_MONTHLY_ANALYSIS_DATA_DOCTYPE)
      .where({ year: year, month: month })
      .indexFields(['year', 'month'])
      .sortBy([{ year: 'desc' }, { month: 'desc' }])
      .limitBy(1)
    const data = await this._client.query(query)

    return data.data
  }

  /**
   * Creates a new EnedisMonthlyAnalysis
   */
  public async createEnedisMonthlyAnalysisData(
    newEnedisMonthlyAnalysisData: EnedisMonthlyAnalysisData
  ): Promise<EnedisMonthlyAnalysisData | null> {
    try {
      const {
        data: EnedisMonthlyAnalysis,
      }: QueryResult<EnedisMonthlyAnalysisData> = await this._client.create(
        ENEDIS_MONTHLY_ANALYSIS_DATA_DOCTYPE,
        newEnedisMonthlyAnalysisData
      )
      return EnedisMonthlyAnalysis
    } catch (error) {
      const errorMessage = `Error creating new EnedisMonthlyAnalysis: ${JSON.stringify(
        error
      )}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(error)
      throw error
    }
  }

  /**
   * Get Max power for a given month and year
   */
  public async getMaxPowerByDate(
    year: number,
    month: number
  ): Promise<MaxPowerEntity[]> {
    const query: QueryDefinition = Q(ENEDIS_MAXPOWER_DOCTYPE)
      .where({ year: year, month: month })
      .indexFields(['year', 'month'])
      .sortBy([{ year: 'desc' }, { month: 'desc' }])
    const data = await this._client.query(query)

    return data.data
  }

  public async getOffPeakHours(): Promise<OffPeakHours[] | null> {
    const accountService = new AccountService(this._client)
    const fluidConfig = new ConfigService().getFluidConfig()
    const account = await accountService.getAccountByType(
      fluidConfig[FluidType.ELECTRICITY].konnectorConfig.slug
    )
    if (!account?.data?.offPeakHours) return null
    return parseOffPeakHours(account.data.offPeakHours)
  }

  public async getOffPeakHoursRatio(
    year: number,
    month: number,
    offPeakHours: OffPeakHours[]
  ): Promise<number> {
    const roundedOffPeakHours = roundOffPeakHours(offPeakHours)
    const splittedOffPeakHours = splitOffPeakHours(roundedOffPeakHours)
    const formattedOffPeakHours = splittedOffPeakHours
      .map(split => formatOffPeakHours(split))
      .join(';')
    logStack(
      'debug',
      `Fetching half-hour consumption for the ranges : ${formattedOffPeakHours}`
    )

    const offPeakHoursConsumption = await this.getOffPeakHourConsumption(
      splittedOffPeakHours,
      year,
      month
    )

    const monthQuery: QueryDefinition = Q(ENEDIS_MONTH_DOCTYPE)
      .where({ year: year, month: month })
      .indexFields(['year', 'month'])
      .limitBy(1)
    const { data: monthDocs } = await this._client.query(monthQuery)
    return offPeakHoursConsumption / monthDocs[0].load
  }

  private async getOffPeakHourConsumption(
    offPeakHours: OffPeakHours[],
    year: number,
    month: number
  ) {
    let minuteDocs: DataloadEntity[] = []

    for (const range of offPeakHours) {
      const { start, end } = range
      const minuteQuery: QueryDefinition = Q(ENEDIS_MINUTE_DOCTYPE)
        .where({
          year: year,
          month: month,
          $and: [
            {
              $or: [
                { hour: { $gt: start.hour } },
                {
                  $and: [
                    { hour: start.hour },
                    { minute: { $gt: start.minute } },
                  ],
                },
              ],
            },
            {
              $or: [
                { hour: { $lt: end.hour } },
                {
                  $and: [{ hour: end.hour }, { minute: { $lte: end.minute } }],
                },
              ],
            },
          ],
        })
        .limitBy(500)
      const { data: docs }: QueryResult<DataloadEntity[]> =
        await this._client.query(minuteQuery)

      minuteDocs = [...minuteDocs, ...docs]

      // if the half-hour 23H30-0H00 is required, add the 0H00-0H30 consumption because the data is shifted in DB
      if (end.hour === 23 && end.minute === 59) {
        const firstHalfHourQuery: QueryDefinition = Q(
          ENEDIS_MINUTE_DOCTYPE
        ).where({
          year: year,
          month: month,
          hour: 0,
          minute: 0,
        })
        const { data: docs }: QueryResult<DataloadEntity[]> =
          await this._client.query(firstHalfHourQuery)

        minuteDocs = [...minuteDocs, ...docs]
      }
    }
    logStack('debug', `Found ${minuteDocs.length} documents`)
    const offPeakHoursConsumption = minuteDocs.reduce((sum: number, doc) => {
      return sum + doc.load
    }, 0)
    return offPeakHoursConsumption
  }
}
