import { QueryResult } from 'cozy-client'
import {
  CustomQuestionType,
  FluidType,
  TimeStep,
  UserQuestionState,
  UserQuizState,
} from 'enums'
import { DateTime } from 'luxon'
import {
  CustomQuestionEntity,
  QuestionEntity,
  QuizEntity,
  UserCustomQuestion,
  UserQuestion,
  UserQuiz,
} from 'models'
import {
  fullMonthGraphData,
  graphData,
  graphMonthData,
} from 'tests/__mocks__/chartData.mock'
import mockClient from 'tests/__mocks__/client.mock'
import {
  UserQuizDone,
  UserQuizReseted,
  UserQuizStarted,
  allQuizEntities,
  customQuestionEntity,
  quizDefault,
  quizEntity,
  userQuiz,
} from 'tests/__mocks__/quizData.mock'
import { getError } from 'tests/__mocks__/testUtils'
import QuizService from './quiz.service'

const localSpy = jest.spyOn(DateTime, 'local')

const randomSpy = jest.spyOn(Math, 'random')

const mockGetGraphData = jest.fn()
const mockGetMaxLoad = jest.fn()
jest.mock('./consumption.service', () => {
  return jest.fn(() => ({
    getGraphData: mockGetGraphData,
    getMaxLoad: mockGetMaxLoad,
  }))
})

describe('Quiz service', () => {
  const quizService = new QuizService(mockClient)

  describe('getAllQuizEntities method', () => {
    it('should return all quiz entity', async () => {
      const mockQueryResult: QueryResult<QuizEntity[]> = {
        data: allQuizEntities,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await quizService.getAllQuizEntities()
      expect(result).toEqual(allQuizEntities)
    })
  })

  describe('getQuizEntityById method', () => {
    it('should return one quiz by id', async () => {
      const mockQueryResult: QueryResult<QuizEntity[]> = {
        data: [quizEntity],
        bookmark: '',
        next: false,
        skip: 0,
      }
      const quizId = 'QUIZ001'
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await quizService.getQuizEntityById(quizId)
      expect(result).toEqual(quizEntity)
    })
  })

  describe('deleteAllQuizEntities method', () => {
    it('should delete all quiz entities', async () => {
      const mockQueryResult: QueryResult<QuizEntity[]> = {
        data: allQuizEntities,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)

      const result = await quizService.deleteAllQuizEntities()
      expect(result).toBeTruthy()
    })

    it('should throw an error because destroy failed', async () => {
      const mockQueryResult: QueryResult<QuizEntity[]> = {
        data: allQuizEntities,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      mockClient.destroy.mockRejectedValue(new Error())
      const error = await getError(async () =>
        quizService.deleteAllQuizEntities()
      )
      expect(error).toEqual(new Error())
    })
  })

  describe('parseQuizEntityToUserQuiz method', () => {
    it('should return the userQuiz from a quizEntity', () => {
      const userQuestions: UserQuestion[] = quizEntity.questions.map(
        (question: QuestionEntity) => {
          return { ...question, result: 0 }
        }
      )
      const mockUpdatedQuiz: UserQuiz = {
        ...quizEntity,
        questions: userQuestions,
        customQuestion: {
          questionLabel: 'Custom1',
          type: CustomQuestionType.DATE,
          timeStep: TimeStep.DAY,
          interval: TimeStep.DAY,
          period: {},
          result: 0,
          singleFluid: false,
        },
        startDate: null,
        state: UserQuizState.UNLOCKED,
        result: 0,
      }

      const result = quizService.parseQuizEntityToUserQuiz(quizEntity)
      expect(result).toEqual(mockUpdatedQuiz)
    })
  })

  describe('getUserQuizfromQuizEntities method', () => {
    it('should return the userQuiz from a quizEntity', () => {
      const searchId = 'QUIZ001'

      const result = quizService.getUserQuizfromQuizEntities(
        allQuizEntities,
        searchId
      )
      expect(result).toEqual(userQuiz)
    })
  })
  describe('endUserQuiz Method', () => {
    it('should return the finished userQuiz', async () => {
      const result = await quizService.endUserQuiz(quizDefault)
      expect(result).toEqual(UserQuizDone)
    })
  })
  describe('startUserQuiz Method', () => {
    it('should return the started userQuiz', async () => {
      jest
        .spyOn(DateTime, 'local')
        .mockReturnValueOnce(
          DateTime.fromISO('2021-01-01T00:00:00.000Z', { zone: 'utc' })
        )
      const result = await quizService.startUserQuiz(quizDefault)
      expect(result).toEqual(UserQuizStarted)
    })
  })
  describe('resetUserQuiz Method', () => {
    it('should return the reseted userQuiz', async () => {
      const result = await quizService.resetUserQuiz(UserQuizDone)
      expect(result).toEqual(UserQuizReseted)
    })
  })
  describe('updateUserQuiz Method', () => {
    it('should return the userQuiz with updated result and correct answer to question', async () => {
      const result = await quizService.updateUserQuiz(userQuiz, true, 0)
      const mockUpdatedQuestion: UserQuestion = {
        ...userQuiz.questions[0],
        result: UserQuestionState.CORRECT,
      }
      userQuiz.questions[0] = mockUpdatedQuestion

      const mockUpdatedQuiz: UserQuiz = {
        ...userQuiz,
        result: 1,
      }
      expect(result).toEqual(mockUpdatedQuiz)
    })
    it('should return the userQuiz with updated result and wrong answer to question', async () => {
      const result = await quizService.updateUserQuiz(userQuiz, false, 2)
      const mockUpdatedQuestion: UserQuestion = {
        ...userQuiz.questions[2],
        result: UserQuestionState.UNCORRECT,
      }
      userQuiz.questions[2] = mockUpdatedQuestion

      const mockUpdatedQuiz: UserQuiz = {
        ...userQuiz,
        result: 0,
      }
      expect(result).toEqual(mockUpdatedQuiz)
    })
    it('should return the userQuiz with updated result and right answer to customQuestion', async () => {
      const result = await quizService.updateUserQuiz(userQuiz, true)
      const mockUpdatedCustomQuestion: UserCustomQuestion = {
        ...userQuiz.customQuestion,
        result: UserQuestionState.CORRECT,
      }
      userQuiz.customQuestion = mockUpdatedCustomQuestion

      const mockUpdatedQuiz: UserQuiz = {
        ...userQuiz,
        result: 1,
      }
      expect(result).toEqual(mockUpdatedQuiz)
    })
  })
  describe('getCustomQuestion Method', () => {
    it('should return the customQuestion for date type without period - What day did I consumme the most on the last week ?', async () => {
      localSpy.mockReturnValue(
        DateTime.fromISO('2020-10-03T00:00:00.000Z', { zone: 'utc' })
      )
      mockGetGraphData.mockResolvedValueOnce(graphData)
      const mockCustomQuestion: CustomQuestionEntity = {
        ...customQuestionEntity,
        interval: TimeStep.WEEK,
      }
      const expected: QuestionEntity = {
        answers: [
          { answerLabel: 'jeudi 1 octobre', isTrue: true },
          { answerLabel: 'vendredi 2 octobre', isTrue: false },
          {
            answerLabel: 'mercredi 30 septembre',
            isTrue: false,
          },
        ],
        explanation:
          'Vous pouvez vérifier cette information sur l’écran Conso.',
        questionLabel: 'Custom1',
        source: '',
      }
      const result = await quizService.getCustomQuestion(mockCustomQuestion, [
        FluidType.ELECTRICITY,
      ])
      expect(result).toEqual(expected)
    })
    it('should return the customQuestion for date type with period - What month did I consumme the most on year 2020 ?', async () => {
      localSpy.mockReturnValue(
        DateTime.fromISO('2020-10-03T00:00:00.000Z', { zone: 'utc' })
      )
      mockGetGraphData.mockResolvedValueOnce(graphMonthData)
      const mockCustomQuestion: CustomQuestionEntity = {
        ...customQuestionEntity,
        timeStep: TimeStep.MONTH,
        interval: TimeStep.YEAR,
        period: { year: 2020 },
      }
      const expected: QuestionEntity = {
        answers: [
          { answerLabel: 'septembre 2020', isTrue: true },
          { answerLabel: 'octobre 2020', isTrue: false },
          { answerLabel: 'août 2020', isTrue: false },
        ],
        explanation:
          'Vous pouvez vérifier cette information sur l’écran Conso.',
        questionLabel: 'Custom1',
        source: '',
      }
      const result = await quizService.getCustomQuestion(mockCustomQuestion, [
        FluidType.ELECTRICITY,
      ])
      expect(result).toEqual(expected)
    })
    it('should return the customQuestion for maxload type without period - Which is your daily max consumption on the last week ?', async () => {
      localSpy.mockReturnValue(
        DateTime.fromISO('2020-10-03T00:00:00.000Z', { zone: 'utc' })
      )
      randomSpy.mockReturnValueOnce(0.9)
      randomSpy.mockReturnValueOnce(0.4)
      mockGetMaxLoad.mockResolvedValueOnce(25)

      const mockCustomQuestion: CustomQuestionEntity = {
        ...customQuestionEntity,
        type: CustomQuestionType.MAXDATA,
        interval: TimeStep.WEEK,
      }
      const expected: QuestionEntity = {
        answers: [
          { answerLabel: '25,00 €', isTrue: true },
          { answerLabel: '3,25 €', isTrue: false },
          {
            answerLabel: '17,50 €',
            isTrue: false,
          },
        ],
        explanation:
          'Vous pouvez vérifier cette information sur l’écran Conso.',
        questionLabel: 'Custom1',
        source: '',
      }
      const result = await quizService.getCustomQuestion(mockCustomQuestion, [
        FluidType.ELECTRICITY,
      ])
      expect(result).toEqual(expected)
    })
    it('should return the customQuestion for maxload type with period - Which is your daily max consumption on january 2020 ?', async () => {
      localSpy.mockReturnValue(
        DateTime.fromISO('2020-10-03T00:00:00.000Z', { zone: 'utc' })
      )
      randomSpy.mockReturnValueOnce(0.9)
      randomSpy.mockReturnValueOnce(0.4)
      mockGetMaxLoad.mockResolvedValueOnce(214)

      const mockCustomQuestion: CustomQuestionEntity = {
        ...customQuestionEntity,
        type: CustomQuestionType.MAXDATA,
        interval: TimeStep.MONTH,
        period: { month: 1, year: 2020 },
      }
      const expected: QuestionEntity = {
        answers: [
          { answerLabel: '214,00 €', isTrue: true },
          { answerLabel: '27,82 €', isTrue: false },
          {
            answerLabel: '149,80 €',
            isTrue: false,
          },
        ],
        explanation:
          'Vous pouvez vérifier cette information sur l’écran Conso.',
        questionLabel: 'Custom1',
        source: '',
      }
      const result = await quizService.getCustomQuestion(mockCustomQuestion, [
        FluidType.ELECTRICITY,
      ])
      expect(mockGetMaxLoad).toHaveBeenCalledWith(
        {
          startDate: DateTime.fromObject({ month: 1, year: 2020 }).startOf(
            'month'
          ),
          endDate: DateTime.fromObject({ month: 1, year: 2020 }).endOf('month'),
        },
        TimeStep.DAY,
        [FluidType.ELECTRICITY],
        undefined,
        true
      )
      expect(result).toEqual(expected)
    })
    it('should return the customQuestion for average type without period - Which is your daily average consumption on the last week ?', async () => {
      localSpy.mockReturnValue(
        DateTime.fromISO('2020-10-03T00:00:00.000Z', { zone: 'utc' })
      )
      randomSpy.mockReturnValueOnce(0.9)
      randomSpy.mockReturnValueOnce(0.4)
      mockGetGraphData.mockResolvedValueOnce(graphData)

      const mockCustomQuestion: CustomQuestionEntity = {
        ...customQuestionEntity,
        type: CustomQuestionType.AVERAGE,
        interval: TimeStep.WEEK,
      }
      const expected: QuestionEntity = {
        answers: [
          { answerLabel: '43,61 €', isTrue: true },
          { answerLabel: '5,67 €', isTrue: false },
          {
            answerLabel: '30,53 €',
            isTrue: false,
          },
        ],
        explanation:
          'Vous pouvez vérifier cette information sur l’écran Conso.',
        questionLabel: 'Custom1',
        source: '',
      }
      const result = await quizService.getCustomQuestion(mockCustomQuestion, [
        FluidType.ELECTRICITY,
      ])
      expect(result).toEqual(expected)
    })
    it('should return the customQuestion for average type with period - Which is your monthly average consumption on 2020 ?', async () => {
      localSpy.mockReturnValue(
        DateTime.fromISO('2020-10-03T00:00:00.000Z', { zone: 'utc' })
      )
      randomSpy.mockReturnValueOnce(0.9)
      randomSpy.mockReturnValueOnce(0.4)
      mockGetGraphData.mockResolvedValueOnce(graphMonthData)

      const mockCustomQuestion: CustomQuestionEntity = {
        ...customQuestionEntity,
        type: CustomQuestionType.AVERAGE,
        timeStep: TimeStep.MONTH,
        interval: TimeStep.YEAR,
        period: { year: 2020 },
      }
      const expected: QuestionEntity = {
        answers: [
          { answerLabel: '43,61 €', isTrue: true },
          { answerLabel: '5,67 €', isTrue: false },
          {
            answerLabel: '30,53 €',
            isTrue: false,
          },
        ],
        explanation:
          'Vous pouvez vérifier cette information sur l’écran Conso.',
        questionLabel: 'Custom1',
        source: '',
      }
      const result = await quizService.getCustomQuestion(mockCustomQuestion, [
        FluidType.ELECTRICITY,
      ])
      expect(mockGetGraphData).toHaveBeenCalledWith(
        {
          startDate: DateTime.fromObject({ year: 2020 }).startOf('year'),
          endDate: DateTime.fromObject({ year: 2020 }).endOf('year'),
        },
        TimeStep.MONTH,
        [FluidType.ELECTRICITY],
        undefined,
        undefined,
        true
      )
      expect(result).toEqual(expected)
    })
    it('should return the customQuestion for average type with weekday period - Which is your monthly average consumption on wednesday ?', async () => {
      localSpy.mockReturnValue(
        DateTime.fromISO('2020-10-03T00:00:00.000Z', { zone: 'utc' })
      )
      randomSpy.mockReturnValueOnce(0.9)
      randomSpy.mockReturnValueOnce(0.4)
      mockGetGraphData.mockResolvedValueOnce(fullMonthGraphData)

      const mockCustomQuestion: CustomQuestionEntity = {
        ...customQuestionEntity,
        type: CustomQuestionType.AVERAGE,
        timeStep: TimeStep.DAY,
        interval: TimeStep.MONTH,
        period: { weekday: 3 },
      }
      const expected: QuestionEntity = {
        answers: [
          { answerLabel: '69,18 €', isTrue: true },
          { answerLabel: '8,99 €', isTrue: false },
          {
            answerLabel: '48,43 €',
            isTrue: false,
          },
        ],
        explanation:
          'Vous pouvez vérifier cette information sur l’écran Conso.',
        questionLabel: 'Custom1',
        source: '',
      }
      const result = await quizService.getCustomQuestion(mockCustomQuestion, [
        FluidType.ELECTRICITY,
      ])
      expect(result).toEqual(expected)
    })
    it('should return the customQuestion for average type with weekday period and singleFluid - Which is your monthly average consumption of #fluid in #unit on wednesday ?', async () => {
      localSpy.mockReturnValue(
        DateTime.fromISO('2020-10-03T00:00:00.000Z', { zone: 'utc' })
      )
      randomSpy.mockReturnValueOnce(0.9)
      randomSpy.mockReturnValueOnce(0.4)
      mockGetGraphData.mockResolvedValueOnce(fullMonthGraphData)

      const mockCustomQuestion: CustomQuestionEntity = {
        ...customQuestionEntity,
        questionLabel: 'Custom1 #fluid #unit',
        type: CustomQuestionType.AVERAGE,
        timeStep: TimeStep.DAY,
        interval: TimeStep.MONTH,
        period: { weekday: 3 },
        singleFluid: true,
      }
      const expected: QuestionEntity = {
        answers: [
          { answerLabel: '69,18 kWh', isTrue: true },
          { answerLabel: '8,99 kWh', isTrue: false },
          {
            answerLabel: '48,43 kWh',
            isTrue: false,
          },
        ],
        explanation:
          'Vous pouvez vérifier cette information sur l’écran Conso.',
        questionLabel: "Custom1 d'électricité kWh",
        source: '',
      }
      const result = await quizService.getCustomQuestion(mockCustomQuestion, [
        FluidType.ELECTRICITY,
      ])
      expect(result).toEqual(expected)
    })
  })
})
